#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import time

import gphoto_2_album
import weibo_2_album
import cached_url
import readee
import yaml
from bs4 import BeautifulSoup
from opencc import OpenCC
from readability import Document
from telegram_util import matchKey, getWid

from .author import _find_author
from .title import _find_title

cc = OpenCC('tw2sp')


class _Article(object):
	def __init__(self, title, author, text, url=None):
		self.title = title
		self.author = author
		self.text = text
		self.url = url


def _find_url(url, soup):
	if 'telegra.ph' not in url:
		return
	address = soup.find('address')
	if not address:
		return
	link = address.find('a')
	return link and link.get('href')


def _trim_webpage(raw):
	anchor = '<!-- detail_toolbox -->'
	index = raw.find(anchor)
	if index != -1:
		return raw[:index]
	return raw


def _get_content_from_album(r, no_text=False):
	result = []
	for url in r.imgs:
		result.append('<img src="%s" />' % url)
	if no_text:
		return '<div><title>%s</title>%s</div>' % (r.title, ''.join(result))
	return '<div><title>%s</title>%s%s</div>' % (r.title, r.cap_html, ''.join(result))


def _get_content(url, force_cache=False):
	if 'weibo.c' in url:
		wid = getWid(url)
		if matchKey(url, ['card', 'ttarticle']):
			new_url = 'https://card.weibo.com/article/m/aj/detail?id=' + wid + '&_t=' + str(int(time.time()))
			json = yaml.load(cached_url.get(new_url, headers={'referer': url}, force_cache=force_cache), Loader=yaml.FullLoader)
			return '<div><title>%s</title>%s</div>' % (json['data']['title'], json['data']['content'])
		return _get_content_from_album(weibo_2_album.get(url))
	if 'photos.google.com/share' in url:
		return _get_content_from_album(gphoto_2_album.get(url), no_text=True)
	return cached_url.get(url, force_cache=force_cache)


def get(url, simplify=False, force_cache=False):
	content = _get_content(url, force_cache=force_cache)
	soup = BeautifulSoup(_trim_webpage(content), 'html.parser')
	article_url = _find_url(url, soup)
	doc = Document(content)
	article = _Article(
		_find_title(soup, doc),
		_find_author(soup),
		readee.export(url, content=content, toSimplified=simplify, list_replace=True),
		article_url)
	if simplify:
		article.title = cc.convert(article.title)
		article.author = cc.convert(article.author)
	return article



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .common import _find_raw_content
from telegram_util import matchKey


def _get_attr_string(attrs):
	if not attrs:
		return ''
	r = []
	for k, v in attrs.items():
		if k in ['content']:
			continue
		r.append(k + ': ' + str(v))
	return '\n'.join(r)


def _yield_possible_author_item(soup):
	yield soup.find("meta", {"name": "byl"})
	yield soup.find("span", {"class": "byline__name"})
	for item in soup.find_all('meta'):
		if 'author' in _get_attr_string(item.attrs):
			yield item
	for item in soup.find_all('div', class_='news_about'):
		yield item.find('p')
	yield soup.find("a", {"id": "js_name"})
	yield soup.find('a', class_='author-url')
	yield soup.find('span', class_='posted-date')
	yield soup.find('a', class_='name')
	yield soup.find('div', class_='article-author')
	yield soup.find("meta", {"name": "application-name"})
	for item in soup.find_all('a'):
		if matchKey(_get_attr_string(item.attrs), ['author']):
			yield item
	for item in soup.find_all('a'):
		if matchKey(_get_attr_string(item.attrs), ['/people/']):
			yield item


def _yield_possible_org_item(soup):
	yield soup.find("meta", {"property": "twitter:site"})
	yield soup.find("meta", {"property": "twitter:domain"})
	yield soup.find("meta", {"property": "og:site_name"})


def _find_possible_raw_content(item_iterator, words_to_ignore=None):
	if words_to_ignore is None:
		words_to_ignore = []
	for item in item_iterator:
		if not item:
			continue
		r = _find_raw_content(item)
		if r and not matchKey(r, words_to_ignore):
			index = r.find(' - ')
			if index == -1:
				return r
			else:
				return r[:index]


def _find_org_name(soup):
	head = str(soup.find('head'))
	if matchKey(head, ['bbc.com']):
		return 'BBC', True
	if matchKey(head, ['nyt.com', 'new york times']):
		return 'NYT', True
	if matchKey(head, ['stackoverflow']):
		return 'StackOverflow', False
	if matchKey(head, ['medium.com']):
		return 'Medium', False
	if matchKey(head, ['dw.come']):
		return 'DW', True
	r = _find_possible_raw_content(_yield_possible_org_item(soup))
	if r:
		return r, False
	return '原文', False


def _find_author(soup):
	author_name = _find_possible_raw_content(
		_yield_possible_author_item(soup),
		['://', 'http', 'www'])
	org, required = _find_org_name(soup)
	if not author_name:
		return org
	if not required or '-' in author_name:
		return author_name
	return author_name + ' - ' + org



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .common import _find_raw_content


def _similar_single(p, media_name):
	return media_name.lower() in p.lower() and (len(p) - len(media_name)) < 10


def _similar(p, media_names):
	return any([_similar_single(p, m) for m in media_names])


def _cleanup_raw_title(raw):
	raw = ''.join(raw.split('BBC Learning English - '))
	media_names = ['nyt', 'new york times', 'stackoverflow', 'bbc', 'opinion']
	index = raw.rfind('- ')
	if index != -1:
		raw = raw[:index]
	raw = raw.strip()
	parts = raw.split('|')
	parts = [p for p in parts if not _similar(p, media_names)]
	return ('|'.join(parts)).strip()


def _yield_possible_title_item(soup):
	yield soup.find("meta", {"property": "twitter:title"})
	yield soup.find("meta", {"name": "twitter:title"})
	yield soup.find("h1", class_='single-post-title')
	yield soup.find("h1", class_='news_title')
	yield soup.find("h1", class_='entry-title')
	yield soup.find('table', class_='infobox')
	yield soup.find('meta', property='og:title')
	yield soup.find("h1", class_='title')
	yield soup.find("h1", class_='story_art_title')
	yield soup.find("h1", class_='post-head')
	yield soup.find("h2", class_='question-title')
	yield soup.find("title")
	yield soup.find("h1")
	yield soup.find("h2")

	for item in soup.find_all('meta'):
		if 'title' in str(item.attrs):
			yield item


def _find_title_from_item(item):
	raw = _find_raw_content(item)
	return _cleanup_raw_title(raw)


def _find_title(soup, doc):
	for item in _yield_possible_title_item(soup):
		if not item:
			continue
		result = _find_title_from_item(item)
		if result and len(result) < 200:
			return result
	return _cleanup_raw_title(doc.title())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

name = 'webpage2telegraph'
token = ''

from html_telegraph_poster import TelegraphPoster
from .article import get


def _get_poster():
	global token
	if token:
		return TelegraphPoster(access_token=token)
	p = TelegraphPoster()
	r = p.create_api_token('webpage2telegraph', 'webpage2telegraph')
	token = r['access_token']
	return p


def _trim_url(url):
	if '://' not in url:
		return url
	loc = url.find('://')
	return url[loc + 3:]


def _format_url(url):
	if '://' not in url:
		return "http://" + url
	return url


def _get_article(url, throw_exception=False, simplify=False, force_cache=False):
	try:
		return article.get(_format_url(url), simplify=simplify, force_cache=force_cache)
	except Exception as e:
		if throw_exception:
			raise e


def get_author_url(article, url, source):
	if not source:
		return ''
	return _format_url(article.url or url)


def get_author_field(author, source):
	if author == '原文' and not source:
		return ''
	return author


def transfer(url, throw_exception=False, source=False, simplify=False, force_cache=False):
	try:
		p = _get_poster()
		article = _get_article(url, throw_exception=throw_exception, simplify=simplify, force_cache=force_cache)
		if not article.text or not article.text.text.strip():  # content is empty
			article.text = '<div>需要手动编辑</div>'
		try:
			r = p.post(
				title=article.title,
				author=get_author_field(article.author, source),
				author_url=get_author_url(article, url, source),
				text=str(article.text))
		except Exception as e:
			if 'CONTENT_TEXT_REQUIRED' in str(e):
				r = p.post(
					title=article.title,
					author=get_author_field(article.author, source),
					author_url=get_author_url(article, url, source),
					text='<div>TO BE ADDED</div>')
			elif 'ACCESS_TOKEN_INVALID' in str(e):
				r = TelegraphPoster().post(
					title=article.title,
					author=get_author_field(article.author, source),
					author_url=get_author_url(article, url, source),
					text=str(article.text))
			else:
				raise e
		return _trim_url(r['url'])
	except Exception as e:
		if throw_exception:
			raise e



import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
	name="webpage2telegraph",
	version="0.0.5",
	author="NullPointerMaker",
	description="Transfer webpage to Telegraph archive.",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://github.com/NullPointerMaker/webpage2telegraph",
	packages=setuptools.find_packages(),
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
		"Operating System :: OS Independent",
	],
	install_requires=[
		'html_telegraph_poster',
		'bs4',
		'readability-lxml',
		'telegram_util',
		'readee',
		'opencc-python-reimplemented',
		'cached_url',
		'weibo_2_album',
		'gphoto_2_album',
	],
	python_requires='>=3.0',
)
